import com.squareup.sqldelight.db.SqlDriver
import com.squareup.sqldelight.sqlite.driver.JdbcSqliteDriver
import controller.*
import fr.louveauAmine.cvApi.dao.CVApiDB
import fr.louveauAmine.cvApi.dao.PROFILE
import io.javalin.Javalin
import io.javalin.apibuilder.ApiBuilder
import io.javalin.core.security.RouteRole

enum class ApiRole : RouteRole { PUBLIC, BASIC_AUTHED }

fun main() {
    val driver: SqlDriver = JdbcSqliteDriver(JdbcSqliteDriver.IN_MEMORY)
    CVApiDB.Schema.create(driver)
    val database = CVApiDB(driver)

    database.profileQueries.insertProfile(PROFILE("Amine Louveau", "truc", "tata", "prout", null, null, null, null))

    val app = Javalin.create{
        it.accessManager { handler, ctx, permittedRoles ->
            val auth = ctx.header("Authorization").orEmpty()
            val isBasicAuthed = auth == System.getenv("LDC_AUTH")
            when {
                permittedRoles.contains(ApiRole.PUBLIC) -> handler.handle(ctx)
                permittedRoles.contains(ApiRole.BASIC_AUTHED) && isBasicAuthed -> handler.handle(ctx)
                else -> ctx.status(401).json("Unauthorized")
            }
        }
    }.start(7070)





    app.routes {
        ApiBuilder.path("api") {
            ApiBuilder.get("me", {
                MeController().getMe()
            }, ApiRole.BASIC_AUTHED)

            ApiBuilder.crud("education/{education-id}", EducationYearController(), ApiRole.BASIC_AUTHED)
            ApiBuilder.crud("interest/{interest-id}", InterestController(), ApiRole.BASIC_AUTHED)
            ApiBuilder.crud("job/{job-id}", JobController(), ApiRole.BASIC_AUTHED)
            ApiBuilder.crud("language/{lang-id}", LanguageController(), ApiRole.BASIC_AUTHED)
            ApiBuilder.crud("profile/{profile-id}", ProfileController(), ApiRole.BASIC_AUTHED)
            ApiBuilder.crud("side-project/{side-id}", SideProjectController(), ApiRole.BASIC_AUTHED)
            ApiBuilder.crud("tag/{tag-id}", TagController(), ApiRole.BASIC_AUTHED)
        }
    }
}